/*
 * Nozzle R Package - JavaScript Library
 *
 * Copyright 2011, Broad Institute / Havard Medical School
 * Authored and maintained by Nils Gehlenborg (nils@hms.harvard.edu)
 */

function initNozzle( reportId, jquery ) {

	var $ = jquery;
	
	var menuHeight = 25;
	var minHorizontalBorder = 30;
	var maxWidth = 1000;
	var autoWidth = false;
	
	var defaultMainToNoteRatio = 0.5;
	var currentMainToNoteRatio = 0.50;
	var mainToNoteDistance = 20;

	var isNoteVisible = false;
	var visibleNoteId = null;
	
			
	var getVerticalMargins = function( element ) { 
		return ( parseInt( element.css( "margin-top" ) ) + parseInt( element.css( "margin-bottom" ) ) );
	};

	var getVerticalPadding = function( element ) { 
		return ( parseInt( element.css( "padding-top" ) ) + parseInt( element.css( "padding-bottom" ) ) );
	};

	var getHorizontalMargins = function( element ) { 
		return ( parseInt( element.css( "margin-left" ) ) + parseInt( element.css( "margin-right" ) ) );
	};

	var getHorizontalPadding = function( element ) { 
		return ( parseInt( element.css( "padding-left" ) ) + parseInt( element.css( "padding-right" ) ) );
	};
	
	
	var getFrame = function() {	
		return $( "#" + reportId + " .frame" );
	}

	var getMain = function() {	
		return $( "#" + reportId + " .main" );
	}

	var getReport = function() {	
		return $( "#" + reportId );
	}
	
	var getMenu = function() {	
		return $( "#" + reportId + " .menu" );
	}
	
	var getVisibleNote = function() {
		return $( visibleNoteId );
	}
	
	
	var getCurrentFrameWidth = function() {
		if ( !autoWidth ) {		
			return ( Math.min( $(window).width() - 2*minHorizontalBorder, maxWidth ) );
		}
		else {
			return ( $(window).width() - 2*minHorizontalBorder );
		}
	};

	var getCurrentFrameLeftBorder = function() {
		if ( !autoWidth ) {		
			return ( ($(window).width() - getCurrentFrameWidth() )/2  );
		}
		else {
			return ( minHorizontalBorder  );
		}
	};

	//$(window).load( function() {
	$(document).ready( function() {
		getFrame().css( 'height', $(window).height() + "px"  );
		getFrame().css( 'width',  getCurrentFrameWidth() + "px"  );
		getFrame().css( 'left',  getCurrentFrameLeftBorder() + "px"  );		
		
		getMenu().css( "height", menuHeight );
		getMain().css( "top", getMenu().outerHeight( true ) );

		if ( visibleNoteId != null )
		{
			getMain().css( "bottom", computeMainBottom() );
			getVisibleNote().css( "top", computeNoteTop() );
			evidenceToggleButton.css( 'top', computeNoteTop() + "px" );			
		}		
	});

	$(window).resize( function() {
		getFrame().css( 'height', $(window).height() + "px"  );
		getFrame().css( 'width',  getCurrentFrameWidth() + "px"  );
		getFrame().css( 'left',  getCurrentFrameLeftBorder() + "px"  );		

		getMenu().css( "height", menuHeight );
		getMain().css( "top", getMenu().outerHeight( true ) );

		if ( visibleNoteId != null )
		{
			getMain().css( "bottom", computeMainBottom() );
			getVisibleNote().css( "top", computeNoteTop() );
			evidenceToggleButton.css( 'top', computeNoteTop() + "px" );			
		}
	});
	
	
	$( "#" + reportId + " .main" ).scroll( function() {
		//console.log( $( "#" + reportId + " .main" ).scrollTop() );
	});

	
	ToggleMode = {
	    TOGGLE: 0,
	    SHOW: 1,
	    HIDE: 2
	};
	
	
	var toggleContents = function( section, animation, mode, target )
	{
		target = target || ".sectionbody";
		
		section.each( function() {
		
			var section = $(this);
			mode = mode || ToggleMode.TOGGLE;					
			
			var button = section.find( ".button.contenttoggle" ).first();
			var body = section.find( target ).first();
			var summarySignificant = section.find( ".summary.significant" ).first();
			
			if ( body.length == 0 )
			{
				body = section;
			}
			
			// get all subsections in this section - if there is only one it will be expanded along with the section
			var subSections = section.find( ".subsection" ).not( "#" + reportId + " .evidence .section, #" + reportId + " .evidence .subsection, #" + reportId + " subsubsection .sectionheader" );
			
			switch ( mode )
			{
				case ToggleMode.TOGGLE:
					if ( subSections.length == 1 )
					{
						if ( !section.hasClass( "visible" ) )
						{
							toggleContents( subSections, false, ToggleMode.SHOW );
						}
					}
					
					section.toggleClass( "visible" );
					animation ? body.slideToggle( "fast" ) : body.toggle();
					if ( summarySignificant !== undefined )
					{
						summarySignificant.toggleClass( "deactivated" );
					}					
					break;
				case ToggleMode.SHOW:
					if ( subSections.length == 1 )
					{
						toggleContents( subSections, false, ToggleMode.SHOW );
					}
					
					section.addClass( "visible" );
					animation ? body.slideDown( "fast" ) : body.show();
					if ( summarySignificant !== undefined )
					{
						summarySignificant.addClass( "deactivated" );
					}					
					break;
				case ToggleMode.HIDE:
					section.removeClass( "visible" );
					animation ? body.slideUp( "fast" ) : body.hide();
					if ( summarySignificant !== undefined )
					{
						summarySignificant.removeClass( "deactivated" );
					}					
					break;
				
			}
			
			button.text( section.hasClass( "visible" ) ? "-" : "+" );					
			button.attr( "title", section.hasClass( "visible" ) ? "Click to collapse" : "Click to expand" );
		} );					
				
	};
	
	var createContentsToggleButtons = function() {
	
		// create button
		var sectionheaders = $( "#" + reportId + " .sectionheader" ).not( ".evidence .sectionheader, .subsubsection .sectionheader" );
		
		sectionheaders.prepend( "<span class=\"button contenttoggle\"><a>" + "-" + "</a></span>" );
		sectionheaders.parent().addClass( "visible" );
		
		var buttons = $( "#" + reportId + " .button.contenttoggle" );
		
		buttons.click( function() {
			toggleContents( $(this).parent().parent(), true, ToggleMode.TOGGLE );
		});										
	};
		   			
	var toggleAllContents = function( animation, mode ) {
		   			
		var sections = $( "#" + reportId + " .section, #" + reportId + " .subsection" ).not( "#" + reportId + " .evidence .section, #" + reportId + " .evidence .subsection, #" + reportId + " subsubsection .sectionheader" );
							
		toggleContents( sections, animation, mode );
	}
	
	var createContentsSummaryBar = function() {
	
		// create button
		var sectionheaders = $( "#" + reportId + " .sectionheader" ).not( "#" + reportId + " .evidence .sectionheader, #" + reportId + " .meta .sectionheader, #" + reportId + " .subsubsection .sectionheader" );
		
		sectionheaders.append( "<span class=\"bar contentssummary\"></span>" );
	};
	
	
	var toggleWidthStateButton = function() {	
		if ( autoWidth ) {
			$( "#menu_width_state" ).text( "Fixed" );
		}		
		else {
			$( "#menu_width_state" ).text( "Auto" );
		}		
	};
	
	var createNozzleMenu = function() {
	
		// TODO: add id???
		var menu = "<div class=\"menu\"><div class=\"menubar\">";

		if ( nozzleNavigation.parentUrl != null ) {
			menu += "<div class=\"menuitem\" id=\"menu_go_parent\" title=\"" + nozzleNavigation.parentName + "\">Up</div>";						
		}
		else {
			menu += "<div class=\"menuitem deactivated\">Up</div>";
		}
		
		if ( nozzleNavigation.previousUrl != null ) {
			menu += "<div class=\"menuitem\" id=\"menu_go_previous\" title=\"" + nozzleNavigation.previousName + "\">&lt;</div>";
		}
		else {
			menu += "<div class=\"menuitem deactivated\">&lt;</div>";
		}

		if ( nozzleNavigation.nextUrl != null ) {
			menu += "<div class=\"menuitem\" id=\"menu_go_next\" title=\"" + nozzleNavigation.nextName + "\">&gt;</div>";
		}
		else {
			menu += "<div class=\"menuitem deactivated\">&gt;</div>";
		}

		menu += "<div class=\"menuitem separator\"></div>";
		
		menu += "<div class=\"menuitem\" id=\"menu_expand_all\">Expand All</div>\
			<div class=\"menuitem\" id=\"menu_collapse_all\">Collapse All</div>\
			<div class=\"menuitem\" id=\"menu_width\">Set&nbsp;<span id=\"menu_width_state\">Auto</span>&nbsp;Width</div>\
			<div class=\"menuitem\" id=\"menu_print_view\">Print Report</div>";
			
		menu += "<div class=\"menuitem separator\"></div>";

		menu += "<div class=\"menuitem highlighted\" id=\"menu_report_problem\" title=\"Report issues related to the content or the functionality/layout of this report.\">Report a Problem</div>";

		menu += "</div></div>";


		getFrame().prepend( menu );
		
		$( "#menu_go_parent" ).click( function() {
			window.location = nozzleNavigation.parentUrl;
		});		

		$( "#menu_go_previous" ).click( function() {
			window.location = nozzleNavigation.previousUrl;
		});

		$( "#menu_go_next" ).click( function() {
			window.location = nozzleNavigation.nextUrl;
		});
				
		$( "#menu_expand_all" ).click( function() {
			toggleAllContents( true, ToggleMode.SHOW );
		});
		
		$( "#menu_collapse_all" ).click( function() {
			toggleAllContents( true, ToggleMode.HIDE );
			reduceAllFigures();
		});

		$( "#menu_width" ).click( function() {		
			autoWidth = !autoWidth;
			toggleWidthStateButton();
			getFrame().animate( { "width": getCurrentFrameWidth() + "px", "left": getCurrentFrameLeftBorder() + "px" }, "slow" );
		});

		$( "#menu_screen_view" ).click( function() {		
			toggleDefaultState( false );
			getFrame().removeClass( "print" );			
			getMain().removeClass( "print" );			
			getReport().removeClass( "print" );			
		});

		$( "#menu_print_view" ).click( function() {
			enlargeAllFigures();		
			toggleAllContents( false, ToggleMode.SHOW );
			getReport().addClass( "print" );			
			getFrame().addClass( "print" );			
			getMain().addClass( "print" );			
			getMenu().addClass( "print" );			
			
			$( "#results_" + reportId + " .evidence" ).not( ".noevidence" ).addClass( "print" );
			$( "#results_" + reportId + " .evidence" ).not( ".noevidence" ).show();
			alert( "This feature is under development.\n\nReload the page to return to the default view after printing." );
		});
		
		$( "#menu_report_problem" ).click( function() {		
			var subject = "Problem Report for \"" + nozzleMeta.reportTitle + "\" (" + nozzleMeta.reportId + ")";			
			var browserDetails = navigator.userAgent;
			var locationDetails = window.location.href;
			var displayDetails = "Not detected."
			
			if (window.screen) {
				displayDetails = 'Current Dimensions: ' + $(window).width() + " x " + $(window).height() + "%0A";
				displayDetails += 'Maximum Dimensions: ' + screen.availWidth + " x " + screen.availHeight + "%0A";
				displayDetails += 'Color Depth: ' + screen.colorDepth;
			}
			
			var message = "Please use this email to submit a problem report related to content or functionality/layout of the \"" + nozzleMeta.reportTitle + "\" report to our JIRA tracking system." + "%0A" +				
				"You may also attach files to your email." + "%0A" +
				"%0A" +
				"We will treat your problem report confidentially. The system details shown below have been included to help us diagnose problems related to the functionality and layout of the report and should not be edited." + "%0A" +
				"%0A" + 
				"Thank you very much for your feedback. It is an important contribution to improve this community resource." + "%0A" +
				"%0A" + 
				"Regards," + "%0A" + 
				"Your Broad GDAC Team (gdac@broadinstitute.org)" + "%0A" + 
				"%0A" + 
				"PS. If this is your first problem report using this email account you will receive a confirmation that a JIRA account has been created for you. You may ignore that message.";
			
			var body = 
				message + "%0A" +
				"%0A" +
				"%0A" +
				"%0A" +
				"== REPORT ======================" + "%0A" +
				"%0A" +
				"URL: " + locationDetails + "%0A" +
				"Title: " + nozzleMeta.reportTitle + "%0A" +
				"Id: " + nozzleMeta.reportId + "%0A" +
				"Creator: " + nozzleMeta.creator + "%0A" +
				"Creation Date: " + nozzleMeta.createDate + "%0A" +
				"Renderer: " + nozzleMeta.renderer + "%0A" +
				"Render Date: " + nozzleMeta.renderDate + "%0A" +
				"%0A" +				
				"%0A" +
				"== MAINTAINER ==================" + "%0A" +
				"%0A" +
				"Name: " + nozzleMeta.maintainer.name + "%0A" +
				"Affiliation: " + nozzleMeta.maintainer.affiliation + "%0A" +
				"Email: " + nozzleMeta.maintainer.email + "%0A" +
				"%0A" +				
				"%0A" +
				"== SYSTEM ======================" + "%0A" +				
				"%0A" +
				"System Time: " + (new Date()) + "%0A" +
				"User Agent: " + browserDetails + "%0A" +
				displayDetails + "%0A" +
				"%0A";
			
			window.location = "mailto:gdac-jira@broadinstitute.org?subject=" + subject + "&body=" + body;
		});
		
		toggleWidthStateButton();		
	};
	
	
	var createEvidenceToggleButtons = function() {
	
		// create button
		var sectionheaders = $( "#" + reportId + " .evidence" );
		
		sectionheaders.prepend( "<span class=\"button evidencetoggle\"><a title=\"Click to hide details\">" + "&times;" + "</a></span>" );
		sectionheaders.addClass( "visible" );
		
		var buttons = $( "#" + reportId + " .evidence .button.evidencetoggle" );
		
		buttons.click( function() {
			toggleContents( getVisibleNote(), false, ToggleMode.HIDE, ".evidence" );
			evidenceToggleButton.appendTo( $( visibleNoteId ) );
			
			var resultId = getVisibleNote().attr( 'id' ).split( '_' )[1];
			$( "#resultid_" + resultId + "_" + reportId ).removeClass( "active" );
			
			var oldScrollTop = $( "#" + reportId + " .main" ).scrollTop();
			
			//console.log( "before: " + $( "#" + reportId + " .main" ).scrollTop() );
			
			
			getMain().animate(
				{
					'bottom': "0px"
				},
				"slow",
				function() {
					isNoteVisible = false;
					visibleNoteId = null;
				});
		});										
	};
	
	createContentsToggleButtons();
	createContentsSummaryBar();
	createEvidenceToggleButtons();
	createNozzleMenu();	
		
	$( "#results_" + reportId + " .evidence" ).hide();
	
	
	var resultLocationRatio = 0.5;
	
	var computeMainBottom = function() {
		return ( getFrame().height() * (1 - currentMainToNoteRatio ) );
	};

	var computeNoteTop = function() {
		return ( getFrame().height() * currentMainToNoteRatio + mainToNoteDistance );
	};
	
	var evidenceToggleButton;
										
	var toggleEvidence = function() {
		var resultId = $(this).attr( 'id' ).split( '_' )[1];
		var evidence = $('#evidenceid_' + resultId + "_" + reportId );
		
		// move the evidence out of the main document and embedded directly in frame
		evidence.appendTo( getFrame() );
				
		if ( evidence.length != 0 )
		{
			if ( visibleNoteId != null && visibleNoteId != '#evidenceid_' + resultId + "_" + reportId ) {
				//toggleContents( $( visibleNoteId ), false, ToggleMode.HIDE, ".evidence" );
			}
			
			$( "#results_" + reportId + " .result" ).not( $(this) ).removeClass( "active" );			
			$(this).toggleClass( 'active' );
			$(this).attr( "title", $(this).hasClass( "active" ) ? "Click to hide details" : "Click to show details" );

			evidence.css( 'top', computeNoteTop() + "px" );			
			evidence.css( 'bottom',  "0px" );			
									
			var offset = $(this).offset().top;
			var scrollTop = $( "#" + reportId + " .main" ).scrollTop();
			var height = $( "#" + reportId + " .main" ).height();
			
			resultLocationRatio = offset/height;
			scrollTarget = scrollTop + (($(window).height()/2) * (resultLocationRatio));
			
			// working, but loss of focus area 
			if ( !isNoteVisible ) {
				getMain().animate({
						"bottom":  computeMainBottom() + "px",					
					},
					"slow",					
					function() {						
						// move the evidence toggle button out of the main document and embedded directly in frame
						evidenceToggleButton = $( '#evidenceid_' + resultId + "_" + reportId + " .button.evidencetoggle" );
						evidenceToggleButton.css( 'top', computeNoteTop() + "px" );
						evidenceToggleButton.css( 'left', "-23px" );

						evidence.fadeToggle( 'slow', function() { evidenceToggleButton.appendTo( getFrame() ); } );							
						$(this).animate( { scrollTop: scrollTarget }, "slow" );				
						
						isNoteVisible = true;
						visibleNoteId = "#evidenceid_" + resultId + "_" + reportId;
				});
			}
			else
			{
				toggleContents( getVisibleNote(), false, ToggleMode.HIDE, ".evidence" );
				evidenceToggleButton.appendTo( $( visibleNoteId ) );

				if ( visibleNoteId != '#evidenceid_' + resultId + "_" + reportId ) {								
					evidence.fadeToggle( 'slow', function() { evidenceToggleButton.appendTo( getFrame() ); } );							
					visibleNoteId = "#evidenceid_" + resultId + "_" + reportId;

					evidenceToggleButton = $( '#evidenceid_' + resultId + "_" + reportId + " .button.evidencetoggle" );
					evidenceToggleButton.css( 'top', computeNoteTop() + "px" );
					evidenceToggleButton.css( 'left', "-23px" );					
				}
				else
				{
					
					getMain().animate(
						{
							'bottom': 0 + "px"
						},
						"slow",
						function() {
							isNoteVisible = false;
							visibleNoteId = null;
						});
				}
			}
		}
		else
		{
			alert( "No evidence found for result \"" + resultId + "\"." );
		}
							
		return false;
	}				
	

	$( "#results_" + reportId + " .result").not( ".noevidence" ).click( toggleEvidence );	
					
	var overallSignificantResults = 0;
	
	$( "#results_" + reportId + " .subsection" ).not( ".evidence" ).each( function( index ) {
		var significantResults = 0;
	
		$(this).find( '.significant' ).each( function( index ) {
			++significantResults;
		});
		
		// TODO: check for unique result ids
		
		if ( significantResults > 0 )
		{
			$(this).addClass( "significant" );
			
			//$(this).find( ".contentssummary" ).first().append( "<span class=\"summary significant\">" + significantResults + " statistically significant findings</span>" );			
			$(this).find( ".contentssummary" ).first().append( "<span class=\"summary significant\" title=\"Section contains findings that crossed a statistical threshold.\"></span>" );						
		}
		
		overallSignificantResults += significantResults;
	} );
					
	if ( overallSignificantResults > 0 )
	{	
		//$( "#results_" + reportId + " .sectionheader .contentssummary" ).not( "#results_" + reportId + " .subsection .sectionheader .contentssummary" ).html( "<span class=\"summary significant\">" + overallSignificantResults + " statistically significant findings</span>" );
		$( "#results_" + reportId + " .sectionheader .contentssummary" ).not( "#results_" + reportId + " .subsection .sectionheader .contentssummary" ).html( "<span class=\"summary significant\" title=\"Section contains findings that crossed a statistical threshold.\"></span>" );
	}
	else
	{
		//$( "#results_" + reportId + " #resultsheader .contentssummary" ).html( "<span class=\"summary\">No significant findings</span>" );
	}
	
	//$( "#summary_" + reportId + " #significantsummary" ).append( overallSignificantResults + " significant results were found." );								
	
	var scaleImage = function() {
		var width = $(this).css( 'width' );
		
		if ( width === "200px" )
		{
			$(this).animate( { 'width': "800px" }, "fast" );
			$(this).attr( "title", "Click to reduce" );
		}
		else
		{
			$(this).animate( { 'width': "200px" }, "fast" );
			$(this).attr( "title", "Click to enlarge" );
		}
							
		return false;
	}				
	
	var enlargeAllFigures = function() {
		$( '.figure .image').each( function() { 
			$(this).find( "img" ).css( "width", "800px" );
			$(this).find( "object" ).css( "width", "800px" );
		});		
	}

	var reduceAllFigures = function() {
		$( '.figure .image').each( function() { 
			$(this).find( "img" ).css( "width", "200px" );
			$(this).find( "object" ).css( "width", "200px" );
		});		
	}
	
	// make figures "scalable"				
	$( '.figure .image').each( function() { 
		$(this).find( "img" ).css( "width", "200px" );
		$(this).find( "img" ).click( scaleImage );
		$(this).find( "img" ).attr( "title", "Click to enlarge" );
	});	
	
	// add parser through the tablesorter addParser method 
	// from: http://stackoverflow.com/questions/4126206/javascript-parsefloat-1-23e-7-gives-1-23e-7-when-need-0-000000123
	// see also here: http://tablesorter.com/docs/example-parsers.html
	$.tablesorter.addParser({ 
		// set a unique id
		id: 'scientificNotation', 
		is: function(s) { 
			return /[+\-]?(?:0|[1-9]\d*)(?:\.\d*)?(?:[eE][+\-]?\d+)?/.test(s); 
		}, 
		format: function(s) { 
			return $.tablesorter.formatFloat(s);
		}, 
		type: 'numeric' 
	});	

	$.tablesorter.addParser({ 
		// set a unique id
		id: 'bigMark', 
		is: function(s) { 
			return /^[0-9]?[0-9]?[0-9](,[0-9][0-9][0-9])*$/.test(s); 
		}, 
		format: function(s) { 
			return s.replace( /,/gi, "" );
		}, 
		type: 'numeric' 
	});	
	
	// make tables sortable
	$( '.resulttable.tablesorter.sortabletable').each( function() {	
			$( this ).tablesorter();	// debug with: $( this ).tablesorter( { debug: true });
	} );

	var toggleDefaultState = function( animate )	{	
		toggleAllContents( animate, ToggleMode.HIDE );	   			
		toggleContents( $( "#overview_" + reportId + ", #summary_" + reportId ), animate );				
		toggleContents( $( "#results_" + reportId ), animate );				
	};
		
	toggleDefaultState( false );
}
